/*
 * PhoneyTV V2.0
 * by Jim  >> BulldogLowell@gmail.com
 * Inspired by Josh >> Deltanu1142@gmail.com
 *
 * This Sketch illuminates 6 sets of LED's in a random fashion as to mimic the 
 * ambient light eminating from a television.  It is intended to make an empty
 * home (or an empty section of a home) appear to be occupied by someone watching
 * TV.  As an alternative to a real television left on, this uses less than 1%
 * of the electrical energy.
 *
 * With the use of the MySensors plugin and gateway, PhoneyTV is intended to 
 * be used with Vera or Raspberry PI.
 *
 * Sketch does not use any delays to create the random blinking as a way to 
 * assure that communication back to the gateway is as unaffected as possible.
 *
 * You can adjust the length of the blink interval and its "twitchyness" by
 * modifying the random number generators, if you prefer more/less 'motion' in
 * in your unit.  The lines are highlighted in the code.
 *
 * Sketch takes advantage of available PWM on pins 3, 5 & 6 using the wht/blu LEDs 
 * to allow fluctuations in the intensity of the light, enhancing the PhoneyTV's
 * natural ambient light effect.
 *
 * This uses the MySensors.org sketch and will attach itself to your gateway as
 * a light, which was another requirement.  Because of that requirement, it 
 * utilizes one digital output pin in the logic (but not physically) and a relay
 * is not actually required to use this sketch.  Also, the radio pinout remains
 * in tact as to preserve the ease of use for MySensor.org users
 *
 * Created 12-APR-2014
 * Free for distrubution
 * Credit should be given to MySensors.org for their base code for relay control
 * and for the radio configuration.  Thanks Guys.
 *
 * 29-May-2014
 * Version 2:  Simplified the code, removing all redundant relay setup from original 
 * code.  Added an on/off momentary pushputton option to be set up on pin 2.  Inproved  
 * the dark dips for longer duration (can be configured) at intervals.
 */
////
#include <Relay.h>
#include <SPI.h>
#include <EEPROM.h> 
#include <RF24.h>
//
#define RELAY_1  3
#define RELAY_ON 0
#define RELAY_OFF 1
//
int pin = 2;
int ledPin3 =  3;      // White using PWM
int ledPin4 =  4;      // Red
int ledPin5 =  5;      // Blue using PWM
int ledPin6 =  6;      // Blue using PWM
int ledPin7 =  7;      // Green
int ledPin8 =  8;      // White (No PWM)
//
boolean ButtonPushed;
boolean timeToDip = false;
int state;
int dipInterval = 10;
int darkTime = 250;
unsigned long currentDipTime;
unsigned long dipStartTime;
unsigned long currentMillis;
int ledState = LOW;
long previousMillis = 0; 
int led = 5;
int interval = 2000;
int twitch = 50;
int dipCount = 0;
int analogLevel = 100;
//
int incomingRelayStatus;
//
Sensor gw;
//
void setup() 
{ 
  pinMode(ledPin3, OUTPUT);
  pinMode(ledPin4, OUTPUT); 
  pinMode(ledPin5, OUTPUT);
  pinMode(ledPin6, OUTPUT);
  pinMode(ledPin7, OUTPUT);
  pinMode(ledPin8, OUTPUT);
  //
  pinMode( pin, OUTPUT);
  attachInterrupt( 0, pushButton, CHANGE); 
  //
  gw.begin();
  gw.sendSketchInfo("PhoneyTV", "1.0");
  gw.sendSensorPresentation(RELAY_1, S_LIGHT);
  gw.getStatus(RELAY_1, V_LIGHT);
  setRelayStatus(gw.getMessage());
}
//
void loop()
{
  getMessage();
  if (ButtonPushed) ChangeLightState();
  //
  if (incomingRelayStatus==1)
  {
    if (timeToDip == false)
    {
      currentMillis = millis();
      if(currentMillis-previousMillis > interval) 
      {
        previousMillis = currentMillis;
        interval = random(750,4001);//Adjusts the interval for more/less frequent random light changes
        twitch = random(40,100);// Twitch provides motion effect but can be a bit much if too high
        dipCount = dipCount++;
      }
      if(currentMillis-previousMillis<twitch)
      {
        led=random(3,9);
        analogLevel=random(50,255);// set the range of the 3 pwm leds
        ledState == LOW ? ledState = HIGH: ledState = LOW; // if the LED is off turn it on and vice-versa:
        switch (led) //for the three PWM pins
        {
        case 3:
          pwmWrite();
          break;
        case 5:
          pwmWrite();
          break;
        case 6:
          pwmWrite();
          break;
        default:
          digitalWrite(led, ledState);
        }
        if (dipCount > dipInterval)
        { 
          timeToDip = true;
          dipCount = 0;
          dipStartTime = millis();
          darkTime = random(50,150);
          dipInterval = random(5,250);// cycles of flicker
        }
      } 
    }
    else
    {
      Serial.println("Dip Time");
      currentDipTime = millis();
      if (currentDipTime - dipStartTime < darkTime)
      {
        for (int i=3;i<9;i++)
        {
          digitalWrite(i,LOW);
        }
      }
      else
      {
        timeToDip = false;

      }
    }
  }
  else
  {
    for (int i=3;i<9;i++)
    {
      digitalWrite(i,LOW);
    }
  }
}
//
void getMessage(){
  if (gw.messageAvailable()) {
    message_s message = gw.getMessage();
    setRelayStatus(message);
  }
}
//
void setRelayStatus(message_s message) {
  if (message.header.messageType=M_SET_VARIABLE && message.header.type==V_LIGHT) 
  {
    incomingRelayStatus = atoi(message.data);
    state = incomingRelayStatus;
    Serial.print("Incoming change for relay on pin:");
    Serial.print(message.header.childId);
    Serial.print(", New status: ");
    Serial.println(incomingRelayStatus);
  }
}
//
void pwmWrite() {
  if (ledState==HIGH){
    analogWrite(led,analogLevel);
  }
  else{
    digitalWrite(led,LOW);
  }
}
//
void pushButton(){ //interrupt with debounce
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 200)
  {
    Serial.println("interrupt");
    ButtonPushed = true;
  }
  last_interrupt_time = interrupt_time;
}
//
void ChangeLightState()
{
  state==1 ? state = 0: state = 1;
  incomingRelayStatus = state;
  gw.sendVariable(RELAY_1, V_LIGHT, state);
  ButtonPushed = false;
}



